#include "bengbox.h"
#include <iostream>
#include <cstdlib>
using std::cerr;
using std::endl;

BengBox::BengBox(int n, double w, double h) {

	walls = 4;
	wall = new Wall[walls];
	wall[0].setX(0);
	wall[1].setX(w);
	wall[2].setY(0);
	wall[3].setY(h); 
	balls = n;
	ball = new Ball[n];
	update = new bool[n];
	events = 4*n+n*(n-1)/2;
	eventtime = new double[events];
}
BengBox::~BengBox() {
	delete [] eventtime;
	delete [] ball;
	delete [] wall;
	delete [] update;
}
const Ball *
BengBox::getBall(int i) const {
	return ball+i;
}
Ball *
BengBox::getBall(int i) {
	return ball+i;
}
unsigned int
BengBox::numBalls() const {
	return balls;
}
void
BengBox::step(double timestep) {
	int i;
	double timeleft = timestep;
	double nextt;

	for (i=0; i<balls; i++) {
		update[i] = true;
	}
	do {
		seteventtimes();
		nextt = timeleft;
		for (i=0; i<events; i++) {
			if (eventtime[i] > 0 && eventtime[i] <= nextt) {
				nextt = eventtime[i];
			}
		}
		move(nextt);
		doevent(nextt);
		timeleft -= nextt;
	} while (timeleft);
}
void
BengBox::seteventtimes() {
	int i,j,n;
	n = 0;
	for (i=0; i<balls; i++) {
		for (j=0; j<4; j++) {
			if (update[i]) {
				eventtime[n]
					= ball[i].getCollideTime(wall[j], n);
			}
			n++;
		}
	}
	for (i=0; i<balls-1; i++) {
		for (j=i+1; j<balls; j++) {
			if (update[i] || update[j]) {
				eventtime[n]
					= ball[i].getCollideTime(ball[j], n);
			}
			n++;
		}
	}
	for (i=0; i<balls; i++) {
		update[i] = false;
	}
}
void
BengBox::doevent(const double t) {
	int i,j,n,count;
	n = 0;
	count = 0;
	for (i=0; i<balls; i++) {
		for (j=0; j<4; j++) {
			if (eventtime[n] == t) {
				ball[i].collide(wall[j], n);
				update[i] = true;
				count++;
			} else {
				eventtime[n] -= t;
			}
			n++;
		}
	}
	for (i=0; i<balls-1; i++) {
		for (j=i+1; j<balls; j++) {
			if (eventtime[n] == t) {
				ball[i].collide(ball[j], n);
				update[i] = true;
				update[j] = true;
				count++;
			} else {
				eventtime[n] -= t;
			}
			n++;
		}
	}
	if (count > 1) {
		cerr << "! " << count << " events at the same time!" <<endl;
	}
}
void
BengBox::move(double t) {
	int i;
	for (i=0; i<balls; i++) {
		ball[i].move(t);
	}
}
void
BengBox::removeOverlap() {
	bool moveBall;
	int i, j, n;
	double w, h;
	double x, y;

	moveBall = false;
	w = wall[1].getX();
	h = wall[3].getY();
	i = n = 0;
	while (i < balls && ++n < 100000) {
		moveBall = false;
		for (j=0; j<balls && !moveBall; j++) {
			if (i != j) {
				moveBall = ball[i].overlap(ball[j]);
			}
		}
		for (j=0; j<walls && !moveBall; j++) {
			moveBall = ball[i].overlap(wall[j]);
		}
		if (moveBall) {
			x = random()*w/RAND_MAX;
			y = random()*h/RAND_MAX;
			ball[i].setPos(x, y);
		} else {
			i++;
		}
	}
//	if (moveBall) {
//		cerr << "still overlap :-(" << endl;
//	}
}
