#include <config.h>
#include "bengsetup.h"
#include "bengsetup.moc"
#include "bengsaver.h"
//#include "bengsaver.xpm"
#include <config.h>
#include <klocale.h>
#include <kglobal.h>
#include <kconfig.h>
#include <kbuttonbox.h>
#include <kaboutdata.h>
#include <kaboutapplication.h>
#include <kcolorbutton.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qcheckbox.h>

BengSetup::BengSetup(QWidget *parent, const char *name, WFlags )
	: QDialog(parent, name, true) {

	readSettings();
	QPushButton *button;
	QLabel *label;
	QSlider *slider;

	setCaption(i18n("Setup Beng Screen Saver"));

	QGridLayout *tl = new QGridLayout(this, 10, 6);
	tl->setColStretch(1, 1);

	label = new QLabel(i18n("Polydispersity:"), this);
	tl->addMultiCellWidget(label, 0, 0, 0, 1);
	slider = new QSlider(this);
	tl->addMultiCellWidget(slider, 1, 1, 0, 1);
	slider->setOrientation(Qt::Horizontal);
	slider->setRange(0,100);
	slider->setValue(int(100*polydispersity));
	connect(slider, SIGNAL(valueChanged(int)),
		this, SLOT(polydispChanged(int)));

	label = new QLabel(i18n("Radius:"), this);
	tl->addMultiCellWidget(label, 2, 2, 0, 1);
	slider = new QSlider(this);
	tl->addMultiCellWidget(slider, 3, 3, 0, 1);
	slider->setOrientation(Qt::Horizontal);
	slider->setRange(1,100);
	slider->setValue(int(100*radius));
	connect(slider, SIGNAL(valueChanged(int)),
		this, SLOT(radiusChanged(int)));

	label = new QLabel(i18n("Number of balls:"), this);
	tl->addWidget(label, 4, 0);
	ballLabel = new QLabel(QString::number(100), this);
	tl->addWidget(ballLabel, 4, 1, Qt::AlignRight);
	slider = new QSlider(this);
	tl->addMultiCellWidget(slider, 5, 5, 0, 1);
	slider->setOrientation(Qt::Horizontal);
	slider->setRange(1,100);
	slider->setValue(int(numberOfBalls));
	connect(slider, SIGNAL(valueChanged(int)),
		this, SLOT(numBallsChanged(int)));

	label = new QLabel(i18n("Number of queens:"), this);
	tl->addWidget(label, 6, 0);
	queenLabel = new QLabel(QString::number(100), this);
	tl->addWidget(queenLabel, 6, 1, Qt::AlignRight);
	queens = new QSlider(this);
	tl->addMultiCellWidget(queens, 7, 7, 0, 1);
	queens->setOrientation(Qt::Horizontal);
	queens->setRange(0,numberOfBalls);
	queens->setValue(numberOfQueens);
	connect(queens, SIGNAL(valueChanged(int)),
		this, SLOT(numQueensChanged(int)));

#ifdef HAVE_GL
	QCheckBox *qcb = new QCheckBox(i18n("3D"), this);
	qcb->setChecked(threeD);
	tl->addMultiCellWidget(qcb, 8, 8, 0, 1);
	connect(qcb, SIGNAL(stateChanged(int)),
		this, SLOT(switch3D(int)));
#endif

	preview = new QWidget(this);
	preview->setFixedSize(220, 170);
	preview->setBackgroundColor(black);
	preview->show();
	saver = new BengSaver(preview->winId());
	tl->addMultiCellWidget(preview, 0, 7, 2, 5);

	KColorButton *cb = new KColorButton(backcolor, this);
	tl->addWidget(cb, 8, 2);
	cb->setMaximumWidth(55); // 1/4th of preview width
	connect(cb, SIGNAL(changed(const QColor &)),
		this, SLOT(backColorChanged(const QColor &)));
	cb = new KColorButton(bigcolor, this);
	tl->addWidget(cb, 8, 3);
	cb->setMaximumWidth(55);
	connect(cb, SIGNAL(changed(const QColor &)),
		this, SLOT(bigColorChanged(const QColor &)));
	cb = new KColorButton(smallcolor, this);
	tl->addWidget(cb, 8, 4);
	cb->setMaximumWidth(55);
	connect(cb, SIGNAL(changed(const QColor &)),
		this, SLOT(smallColorChanged(const QColor &)));
	cb = new KColorButton(queencolor, this);
	tl->addWidget(cb, 8, 5);
	cb->setMaximumWidth(55);
	connect(cb, SIGNAL(changed(const QColor &)),
		this, SLOT(queenColorChanged(const QColor &)));

	KButtonBox *bbox = new KButtonBox(this);
	button = bbox->addButton(i18n("&About"));
	connect(button, SIGNAL(clicked()), this, SLOT(about()));
	bbox->addStretch(1);
	
	button = bbox->addButton(i18n("&OK"));
	button->setFocus();
	connect(button, SIGNAL(clicked()), this, SLOT(okPressed()));

	button = bbox->addButton(i18n("&Cancel"));
	connect(button, SIGNAL(clicked()), this, SLOT(reject()));
	bbox->layout();
	tl->addMultiCellWidget(bbox, 9, 9, 0, 5);
	ballLabel->setText(QString::number(numberOfBalls));
	queenLabel->setText(QString::number(numberOfQueens));
}
void
BengSetup::readSettings() {
	KConfig *config = KGlobal::config();
	config->setGroup("Settings");

	numberOfBalls = config->readNumEntry("Number of Balls", 5);
	polydispersity = config->readDoubleNumEntry("Polydispersity", 0.5);
	radius = config->readDoubleNumEntry("Radius", 0.5);
	numberOfQueens = config->readNumEntry("Number of Queens", 0);
	backcolor = config->readColorEntry("Background color", &green);
	bigcolor = config->readColorEntry("Big ball color", &white);
	smallcolor = config->readColorEntry("Small ball color", &black);
	queencolor = config->readColorEntry("Queen ball color", &red);
	threeD = config->readBoolEntry("3D", true);
}
// Ok pressed: save and exit
void
BengSetup::okPressed() {
	KConfig *config = KGlobal::config();
	config->setGroup("Settings");

	config->writeEntry("Number of Balls", numberOfBalls);
	config->writeEntry("Polydispersity", polydispersity);
	config->writeEntry("Radius", radius);
	config->writeEntry("Number of Queens", numberOfQueens);
	config->writeEntry("Background color", backcolor);
	config->writeEntry("Big ball color", bigcolor);
	config->writeEntry("Small ball color", smallcolor);
	config->writeEntry("Queen ball color", queencolor);
	config->writeEntry("3D", threeD);
	config->sync();

	accept(); // leave dialog
}
void
BengSetup::polydispChanged(int p) {
	polydispersity = p/100.0;
	saver->setPolydispersity(polydispersity);
}
void
BengSetup::radiusChanged(int r) {
	radius = r/100.0;
	saver->setRadius(radius);
}
void
BengSetup::numBallsChanged(int b) {
	numberOfBalls = b;
	saver->setNumBalls(b);
	ballLabel->setText(QString::number(numberOfBalls));
	queens->setMaxValue(b);
	queenLabel->setText(QString::number(numberOfQueens));
}
void
BengSetup::numQueensChanged(int q) {
	numberOfQueens = q;
	saver->setQueens(q);
	queenLabel->setText(QString::number(numberOfQueens));
}
void
BengSetup::about() const {
	KAboutData aboutData( "bengsaver.kss", "BengSaver", VERSION,
	I18N_NOOP( "A screensaver that has either bouncing balls\n"
		" or a number of swarms with collision detection." ),
		KAboutData::License_GPL_V2,
		"(c) 2003, Jos van den Oever", 0,
		"http://www.vandenoever.info/", "jos@vandenoever.info");
	aboutData.addAuthor("Jos van den Oever", "Author and maintainer",
		"jos@vandenoever.info", "http://www.vandenoever.info/" );
//	aboutData.addIcon(bengsaver_xpm);
//	aboutData.addCredit( "", "", 0, 0 );
	KAboutApplication about(&aboutData);
	about.exec();
}
void
BengSetup::backColorChanged(const QColor &c) {
	backcolor = c;
	saver->setBackColor(c);
}
void
BengSetup::bigColorChanged(const QColor &c) {
	bigcolor = c;
	saver->setBigColor(c);
}
void
BengSetup::smallColorChanged(const QColor &c) {
	smallcolor = c;
	saver->setSmallColor(c);
}
void
BengSetup::queenColorChanged(const QColor &c) {
	queencolor = c;
	saver->setQueenColor(c);
}
void
BengSetup::switch3D(int state) {
	threeD = state == QButton::On;
	saver->set3D(threeD);
}
