#include "glbox.h"
#ifdef HAVE_GL
#include <GL/glu.h>
#include <iostream>
using std::cout;
using std::endl;
/*!
  Create a GLBox widget
*/
GLBox::GLBox( QWidget* parent, const char* name )
    : QGLWidget( parent, name ) {

	// create a quadric object
	makeCurrent(); // needs to be called before any OpenGL API call
			// that's not in paintGL, initializeGl or resizeGL
	gobj = gluNewQuadric();
	gluQuadricDrawStyle(gobj, GLU_FILL);
}
/*!
  Release allocated resources
*/
GLBox::~GLBox() {
	// delete the quadric object
	makeCurrent();
	gluDeleteQuadric(gobj);
}
/*!
  Paint the box. The actual openGL commands for drawing the box are
  performed here.
*/
void GLBox::paintGL() { //Qt calls automatically
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	gluLookAt(0.0, 0.0, 10.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

	// transform in such a way that the box coordinates go from
	// 0->width(), 0->height(), 0->depth
	glScalef(1, 1, -1);
	glTranslatef( -width()/2.0, -height()/2.0, zpos);

	// define the colors
	GLfloat mat_Y[] = {backcolor.red()/255.0, backcolor.green()/255.0,
			backcolor.blue()/255.0};
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_Y);

	// draw box
	glBegin(GL_POLYGON); // start drawing a polygon
	glNormal3f(0.0, 0.0, -1.0);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_Y);
	glVertex3f(0.0, 0.0, depth); // point 1 of polygon
	glVertex3f(width(), 0.0, depth); // point 2 of polygon
	glVertex3f(width(), height(), depth); // point 3 of polygon
	glVertex3f(0.0, height(), depth); // point 4 of polygon
	glEnd(); // end drawing polygon
	glBegin(GL_POLYGON); // start drawing a polygon
	glNormal3f(0.0, 1.0, 0);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_Y);
	glVertex3f(0.0, 0.0, 0); // point 1 of polygon
	glVertex3f(width(), 0.0, 0); // point 2 of polygon
	glVertex3f(width(), 0.0, depth); // point 3 of polygon
	glVertex3f(0.0, 0.0, depth); // point 4 of polygon
	glEnd(); // end drawing polygon
	glBegin(GL_POLYGON); // start drawing a polygon
	glNormal3f(0.0, 1.0, 0);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_Y);
	glVertex3f(0.0, height(), 0); // point 1 of polygon
	glVertex3f(width(), height(), 0); // point 2 of polygon
	glVertex3f(width(), height(), depth); // point 3 of polygon
	glVertex3f(0.0, height(), depth); // point 4 of polygon
	glEnd(); // end drawing polygon
	glBegin(GL_POLYGON); // start drawing a polygon
	glNormal3f(1.0, 0, 0);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_Y);
	glVertex3f(0.0, 0.0, 0); // point 1 of polygon
	glVertex3f(0.0, height(), 0); // point 2 of polygon
	glVertex3f(0.0, height(), depth); // point 3 of polygon
	glVertex3f(0.0, 0.0, depth); // point 4 of polygon
	glEnd(); // end drawing polygon
	glBegin(GL_POLYGON); // start drawing a polygon
	glNormal3f(1.0, 0, 0);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_Y);
	glVertex3f(width(), 0.0, 0); // point 1 of polygon
	glVertex3f(width(), height(), 0); // point 2 of polygon
	glVertex3f(width(), height(), depth); // point 3 of polygon
	glVertex3f(width(), 0.0, depth); // point 4 of polygon
	glEnd(); // end drawing polygon
/*
	// draw reference spheres
	GLfloat radius = (width()<height() ?width() :height()) /10.0;
	// put sphere near origin
	glPushMatrix(); // save current matrix
	glTranslatef(radius, radius, radius); // move to position of object
	gluSphere(gobj, radius, 30, 15);
	glPopMatrix(); // restore last matrix
	glPushMatrix();
	glTranslatef(width()-radius, radius, radius);
	gluSphere(gobj, radius, 30, 15);
	glPopMatrix();
	glPushMatrix();
	glTranslatef(radius, height()-radius, radius);
	gluSphere(gobj, radius, 30, 15);
	glPopMatrix();
	glPushMatrix();
	glTranslatef(width()-radius, height()-radius, radius);
	gluSphere(gobj, radius, 30, 15);
	glPopMatrix();
	glPushMatrix(); // save current matrix
	glTranslatef(radius, radius, depth-radius); // move to position of object
	gluSphere(gobj, radius, 30, 15);
	glPopMatrix(); // restore last matrix
	glPushMatrix();
	glTranslatef(width()-radius, radius, depth-radius);
	gluSphere(gobj, radius, 30, 15);
	glPopMatrix();
	glPushMatrix();
	glTranslatef(radius, height()-radius, depth-radius);
	gluSphere(gobj, radius, 30, 15);
	glPopMatrix();
	glPushMatrix();
	glTranslatef(width()-radius, height()-radius, depth-radius);
	gluSphere(gobj, radius, 30, 15);
	glPopMatrix();
*/
	double maxr = 0;
	for (uint i=0; i<balls.size(); ++i) {
		if (balls[i].radius > maxr) {
			maxr = balls[i].radius;
		}
	}
	for (uint i=0; i<balls.size(); ++i) {
		mat_Y[0] = balls[i].c.red()/255.0;
		mat_Y[1] = balls[i].c.green()/255.0;
		mat_Y[2] = balls[i].c.blue()/255.0;
		glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_Y);
		glPushMatrix();
		glTranslatef(balls[i].x, balls[i].y, maxr);
		double r = balls[i].radius;
		gluSphere(gobj, r, 30, 15);
		glPopMatrix();
	}

	glFlush(); // flush all commands to screen
}
/*!
  Set up the OpenGL rendering state, and define display list
*/
void GLBox::initializeGL() { //Qt calls automatically
	GLfloat mat_shininess[] = { 50.0 };
	GLfloat light0_position[] = { 1.0, 2.0, 5.0, 0.0 };
	GLfloat light1_position[] = { -1.0, 2.0, -5.0, 0.0 };
	GLfloat white_light[] = {1.0, 1.0, 1.0, 1.0};
	GLfloat lmodel_ambient[] = {0.1, 0.1, 0.1, 1.0};
	glShadeModel( GL_SMOOTH );

	// specify material parameters for the lighting model
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	// specify light source parameters for two lights
	glLightfv(GL_LIGHT0, GL_POSITION, light0_position);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, white_light);
	glLightfv(GL_LIGHT0, GL_SPECULAR, white_light);
	glLightfv(GL_LIGHT1, GL_POSITION, light1_position);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, white_light);
	glLightfv(GL_LIGHT1, GL_SPECULAR, white_light);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);
	glEnable(GL_DEPTH_TEST);
}
/*!
  Set up the OpenGL view port, matrix mode, etc.
*/
void GLBox::resizeGL( int w, int h ) { //Qt calls automatically
	depth = (w < h) ?h :w /2.0;
	zpos = 200*depth;

	glViewport( 0, 0, (GLint)w, (GLint)h );
	// specify which matrix is the current matrix
	glMatrixMode( GL_PROJECTION );
	// replace the current matrix with the identity matrix
	glLoadIdentity();
	// multiply the current matrix by a perspective matrix
	glFrustum( -w/2.0, w/2.0, -h/2.0, h/2.0, zpos, (zpos+depth));
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
}
void
GLBox::setBackgroundColor ( const QColor & c ) {
	backcolor = c;
	makeCurrent(); // needs to be called before any OpenGL API call
	glClearColor(c.red()/255.0,c.green()/255.0,c.blue()/255.0, 0.0 );
	updateGL();
}
void
GLBox::setNumBalls(uint n) {
	balls.resize(n);
}
uint
GLBox::getNumBalls() const {
	return balls.size();
}
void
GLBox::setBallCoord(uint n, double x, double y) {
	balls[n].x = x;
	balls[n].y = height()-y;
}
void
GLBox::setBallRadius(uint n, double r) {
	balls[n].radius = r;
}
void
GLBox::setBallColor(uint n, QColor c) {
	balls[n].c = c;
}
#endif
